﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace EduNetworkBuilder
{
    public partial class VLANConfig : Form
    {

        NetworkDevice TheDevice = null;
        BindingList<VLANName> VLANNames = new BindingList<VLANName>();
        DataTable VlanBindings = new DataTable();
        List<NetworkInterface> interfaces = new List<NetworkInterface>();

        public VLANConfig(NetworkDevice ToEdit)
        {
            InitializeComponent();
            TheDevice = ToEdit;

            LocalSetup();
        }

        private void LocalSetup()
        {
            //Bind the vlan names
            Network theNet = NB.GetNetwork();
            //var _VLANNames = from row in theNet.VlanNames select new { VLAN_ID = row.Key, VLAN_Name = row.Value };
            //dgv_VLANNames.DataSource = _VLANNames.ToArray();

            theNet.VlanNames.Sort((x, y) => x.ID.CompareTo(y.ID));

            dgv_VLANNames.DataSource = theNet.VlanNames;
            //dgv_VLANNames.Columns.Add("ID", "ID");
            //dgv_VLANNames.Columns.Add("Name", "Name");
            dgv_VLANNames.AllowUserToAddRows = true;

            VlanBindings.Clear();
            VlanBindings.Columns.Add("IF", typeof(string));
            foreach(VLANName vn in theNet.VlanNames)
            {
                VlanBindings.Columns.Add(vn.Name, typeof(string));
            }

            List<string> theNics = TheDevice.NICNames();
            foreach(string oneNic in theNics)
            {
                NetworkCard NIC = TheDevice.NicFromName(oneNic);
                if (NIC.NicName() == NicType.lo.ToString()) continue;
                if (NIC.NicName() == NicType.management_interface.ToString()) continue;
                for (int i=0; i< NIC.IFCount; i++)
                {
                    NetworkInterface nif = NIC.GetInterface(i);                    

                    DataRow DR = VlanBindings.NewRow();
                    DR["IF"] = nif.nic_name;
                    foreach (VLANName vn in theNet.VlanNames)
                    {
                        DR[vn.Name] = nif.GetVLANTag(vn.ID).ToString();
                    }
                    VlanBindings.Rows.Add(DR);
                }
            }
            dgv_VLANAssignments.AutoGenerateColumns = false;

            //dgv_VLANAssignments.Columns.Add("IF","IF");
            DataGridViewTextBoxColumn scol = new DataGridViewTextBoxColumn();
            scol.ValueType = typeof(string);
            scol.DataPropertyName = "IF";
            scol.Name = "IF";
            scol.HeaderText = "Interface";
            scol.ReadOnly = true;
            scol.Frozen = true;
            dgv_VLANAssignments.Columns.Add(scol);

            List<string> enumvalues = new List<string>();
            foreach (VLANTagType vtt in Enum.GetValues(typeof(VLANTagType)))
                enumvalues.Add(vtt.ToString());

            foreach (VLANName vn in theNet.VlanNames)
            {
                DataGridViewComboBoxColumn col = new DataGridViewComboBoxColumn();
                col.Name = vn.Name;
                col.DataPropertyName = vn.Name;
                col.DataSource = enumvalues;
                col.Width = 75;
                dgv_VLANAssignments.Columns.Add(col);
            }

            dgv_VLANAssignments.DataSource = VlanBindings;
        }


        private void btnDone_Click(object sender, EventArgs e)
        {
            //We will probably do a lot more than this later.
            Close();
        }

        private void dgv_VLANNames_CellParsing(object sender, DataGridViewCellParsingEventArgs e)
        {
        //    BindingList<VLANName> tempitem = new BindingList<VLANName>();
            VLANName current = ((BindingList<VLANName>)dgv_VLANNames.DataSource)[e.RowIndex];
            string key = current.ID.ToString();
            int id = current.ID;
            string value = current.Name;
            string cellValue = e.Value.ToString();
            if (e.ColumnIndex == 0) key = cellValue;
            else value = cellValue;
            ((BindingList<VLANName>)dgv_VLANNames.DataSource)[e.RowIndex] = new VLANName { ID = id, Name = value };
        }

    }
}
